import pygame
import random

class snowflake():
    def __init__(self, pos, speed=2):
        self.pos = [pos, 0] # [x, y] position. Starts at the top of the screen
        self.speed = speed
        self.color = (255,255,255)
        self.size = 5

    def update(self):
        self.pos[1] += self.speed
    
    def get_pos(self):
        return self.pos

    def get_size(self):
        return self.size
    
    def get_color(self):
        return self.color

class Cart():
    def __init__(self, pos = 5):
        self.size = [60, 40]
        self.pos = [pos, world_h - self.size[1] - floor]
        self.color = (255,255,255)
        self.velocity = 0
        self.acceleration = 0
    
    def get_pos(self):
        return self.pos
    
    def get_color(self):
        return self.color
    
    def get_size(self):
        return self.size

    def set_position(self, x_position):
        self.velocity = 0
        self.acceleration = 0
        self.pos = [x_position, world_h - self.size[1] - floor]
    
    def move(self, direction):
        if direction == 'left':
            self.acceleration = -1 / 20
        elif direction == 'right':
            self.acceleration = 1 / 20
        elif direction == None:
            if self.velocity < -0.04:
                self.acceleration = 1 / 40
            elif self.velocity > 0.04:
                self.acceleration = -1 / 40
            else:
                self.velocity = 0
                self.acceleration = 0
    
    def update(self, max_velocity = 10):
        if self.velocity >= max_velocity:
            self.velocity = max_velocity
        self.velocity += self.acceleration
        self.pos[0] += self.velocity
        if self.pos[0] < 0:
            self.pos[0] = 0
            self.velocity = 0
            self.acceleration = 0
        elif self.pos[0] + self.size[0] > world_w:
            self.pos[0] = world_w - self.size[0]
            self.velocity = 0
            self.acceleration = 0

#Initialize Game
pygame.init()

#Set up clock
clock = pygame.time.Clock()
FPS = 90

#Set up fonts
score_font = pygame.font.SysFont("Comic Sans", 32)
minutes_font = pygame.font.SysFont("Comic Sans", 64)
seconds_font = pygame.font.SysFont("Comic Sans", 64)
type_font = pygame.font.SysFont("Ariel", 46)
countdown_font = pygame.font.SysFont("Comic Sans", 90)

#Set up display
world_w, world_h = 1200, 770 #1000, 670
screen = pygame.display.set_mode((world_w, world_h))
pygame.display.set_caption("SNOW GAME")
background = (30, 30, 30)

floor = 8
floor_color = (0, 230, 255)
reset = True
while reset:
    pointer_timer = 0
    winning_screen = True
    pointer_bool = True
    getting_name = True
    running = True
    name = ""
    pointer_position = world_w // 5

    while getting_name:
        text = ""
        screen.fill(background)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                getting_name = False
                running = False
                winning_screen = False
                reset = False

            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_q:
                    text = "q"
                if event.key == pygame.K_w:
                    text = "w"
                if event.key == pygame.K_e:
                    text = "e"
                if event.key == pygame.K_r:
                    text = "r"
                if event.key == pygame.K_t:
                    text = "t"
                if event.key == pygame.K_y:
                    text = "y"
                if event.key == pygame.K_u:
                    text = "u"
                if event.key == pygame.K_i:
                    text = "i"
                if event.key == pygame.K_o:
                    text = "o"
                if event.key == pygame.K_p:
                    text = "p"
                if event.key == pygame.K_a:
                    text = "a"
                if event.key == pygame.K_s:
                    text = "s"
                if event.key == pygame.K_d:
                    text = "d"
                if event.key == pygame.K_f:
                    text = "f"
                if event.key == pygame.K_g:
                    text = "g"
                if event.key == pygame.K_h:
                    text = "h"
                if event.key == pygame.K_j:
                    text = "j"
                if event.key == pygame.K_k:
                    text = "k"
                if event.key == pygame.K_l:
                    text = "l"
                if event.key == pygame.K_z:
                    text = "z"
                if event.key == pygame.K_x:
                    text = "x"
                if event.key == pygame.K_c:
                    text = "c"
                if event.key == pygame.K_v:
                    text = "v"
                if event.key == pygame.K_b:
                    text = "b"
                if event.key == pygame.K_n:
                    text = "n"
                if event.key == pygame.K_m:
                    text = "m"
                if event.key == pygame.K_PERIOD:
                    text = "."
                if event.key == pygame.K_BACKSPACE:
                    text = "DELETE"
                if event.key == pygame.K_RETURN:
                    text = "ENTER"

        if text == "DELETE" and name:
            deleted_character = name[-1]
            name = name[:-1]
            if deleted_character == ".":
                pointer_position -= 10
            elif deleted_character in ('i', 'l'):
                pointer_position -= 10
            elif deleted_character in ('t', 'r'):
                pointer_position -= 14
            elif deleted_character in ('w','m'):
                pointer_position -= 26
            else:
                pointer_position -= 19

        elif text == ".":
            pointer_position += 10
            name += text
        elif text in ('i', 'l'):
            pointer_position += 10
            name += text
        elif text in ('t', 'r'):
            pointer_position += 14
            name += text
        elif text in ('w','m'):
            pointer_position += 26
            name += text
        elif text != 'ENTER' and text != 'DELETE' and text:
            pointer_position += 19
            name += text
        elif text != 'DELETE' and text:
            getting_name = False
        
        if pointer_timer <= 0:
            pointer_timer = 30
            pointer_bool = not pointer_bool
        else:
            pointer_timer -= 1
        
        pygame.draw.rect(screen, (255, 255, 255), (world_w // 5 - 10, world_h // 2 - 35, world_w // 5 * 3 + 15, 70))
        pygame.draw.rect(screen, background, (world_w // 5 - 5, world_h // 2 - 30, world_w // 5 * 3 + 5, 60))
        if pointer_bool:
            pygame.draw.rect(screen, (255, 255, 255), (pointer_position, world_h // 2 - 20, 3, 40))

        type_text = type_font.render(f"{name}", True, (255, 255, 255))
        type_text_box = type_text.get_rect()
        type_text_box.midleft = (world_w // 5, world_h//2)
        screen.blit(type_text, type_text_box)

        pygame.display.update()
        clock.tick(FPS)

    #Set up constant variables:
    testing = True
    creation_timer = 0
    flakes = []
    player = Cart()
    direction = []
    score = 0
    timer = FPS
    minutes = 0
    seconds = 30
    pad = ""
    while running:
        screen.fill(background)
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                running = False
                winning_screen = False
                reset = False
            
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    running = False
                    winning_screen = False
                if event.key == pygame.K_a or event.key == pygame.K_LEFT:
                    direction.append('left')
                if event.key == pygame.K_d or event.key == pygame.K_RIGHT:
                    direction.append('right')
                if event.key == pygame.K_RETURN:
                    testing = False
                    player.set_position(5)
                    flakes.clear()
                    creation_timer = 0
                    score = 0
                    countdown = FPS * 3
                    number = 3
                    while countdown:
                        screen.fill(background)
                        for event in pygame.event.get():
                            if event.type == pygame.QUIT:
                                countdown = 1
                                running = False
                                winning_screen = False
                                reset = False

                        countdown -= 1
                        if countdown < FPS * 2:
                            number = 2
                        if countdown < FPS:
                            number = 1

                        countdown_text = countdown_font.render(f"Starting in: {number}", True, (255, 255, 255))
                        countdown_text_box = countdown_text.get_rect()
                        countdown_text_box.center = (world_w // 2, world_h // 2)
                        screen.blit(countdown_text, countdown_text_box)
                    
                        clock.tick(FPS)
                        pygame.display.update()
            
            if event.type == pygame.KEYUP:
                if event.key == pygame.K_a or event.key == pygame.K_LEFT:
                    if 'left' in direction:
                        direction.remove('left')
                if event.key == pygame.K_d or event.key == pygame.K_RIGHT:
                    if 'right' in direction:
                        direction.remove('right')

        if direction:
            player.move(direction[-1])
        else:
            player.move(None)
        player.update()
        pygame.draw.rect(screen, player.get_color(), (player.get_pos()[0], player.get_pos()[1], player.get_size()[0], player.get_size()[1]))
        
        if creation_timer <= 0:
            flake = snowflake(random.randint(20, world_w - 20), random.choice([1.9, 2, 2.1]))
            flakes.append(flake)
            creation_timer = 20 #Original was 45, FPS/2
        
        for flake in flakes:
            flake.update()
            if flake.get_pos()[1] > world_h:
                flakes.remove(flake)
            
            if flake.get_pos()[0] + flake.get_size() > player.get_pos()[0] and flake.get_pos()[0] - flake.get_size() < player.get_pos()[0] + player.get_size()[0]:
                if flake.get_pos()[1] + flake.get_size() >= player.get_pos()[1] and flake.get_pos()[1] - flake.get_size() <= player.get_pos()[1] + player.get_size()[1]:
                    if flake in flakes:
                        flakes.remove(flake)
                    score += 1

            pygame.draw.circle(screen, flake.get_color(), (flake.get_pos()[0], flake.get_pos()[1]), flake.get_size())

        if not testing:
            if seconds < 0:
                if minutes == 0:
                    running = False
                else:
                    minutes -= 1
                    seconds = 59
            
            if timer <= 0:
                seconds -= 1
                timer = FPS
            else:
                timer -= 1

            score_text = score_font.render(f"{score}", True, (255, 0, 0))
            score_text_box = score_text.get_rect()
            score_text_box.center = (player.get_pos()[0] + player.get_size()[0] // 2, player.get_pos()[1] + player.get_size()[1] // 2)
            screen.blit(score_text, score_text_box)

            minutes_text = minutes_font.render(f"{minutes}:", True, (0, 210, 255))
            minutes_text_box = minutes_text.get_rect()
            minutes_text_box.midright = (100, 100)
            screen.blit(minutes_text, minutes_text_box)

            if len(str(seconds)) == 1:
                pad = "0"
            else:
                pad = ""

            seconds_text = seconds_font.render(f"{pad}{seconds}", True, (0, 210, 255))
            seconds_text_box = seconds_text.get_rect()
            seconds_text_box.midleft = (102, 102)
            screen.blit(seconds_text, seconds_text_box)

        creation_timer -= 1
        
        pygame.draw.rect(screen, floor_color, (0, world_h - floor, world_w, floor))

        pygame.display.update()
        clock.tick(FPS)

    end_font = pygame.font.SysFont("Comic Sans", 100)
    end_text = end_font.render(f"Your Score: {score}", True, (255, 0, 0))
    end_text_box = end_text.get_rect()
    end_text_box.center = (world_w // 2, 150)

    check = True
    while winning_screen:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                winning_screen = False
                reset = False
            
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_RETURN:
                    winning_screen = False
        
        screen.fill(background)

        if check:
            check = False
            if name:
                with open("Scores.txt", 'a') as file:
                    file.write(f'{name}: {score}\n')

        if creation_timer <= 0:
            flake = snowflake(random.randint(20, world_w - 20), random.choice([1.9, 2, 2.1]))
            flakes.append(flake)
            creation_timer = 45
        
        for flake in flakes:
            flake.update()
            if flake.get_pos()[1] > world_h:
                flakes.remove(flake)
            
            pygame.draw.circle(screen, flake.get_color(), (flake.get_pos()[0], flake.get_pos()[1]), flake.get_size())
        
        
        scores = {}
        with open("Scores.txt", "r") as file:
            for line in file:
                line = line.strip('\n')
                split_line = line.split()
                hs = int(split_line[1])
                player = split_line[0]
                scores[player] = hs
        
        scores = dict(sorted(scores.items(), key=lambda item: item[1], reverse=True)) #Idk how this works but yea it does thanks stackoverflow

        i = 0
        for player in scores:
            if i < 5:
                list_font = pygame.font.SysFont("Comic Sans", 60)
                list_text = list_font.render(f"{player} {scores[player]}", True, (0, 210, 255))
                list_text_box = list_text.get_rect()
                list_text_box.center = (world_w // 2, 275 + 80*i)
                screen.blit(list_text, list_text_box)
            i += 1

        creation_timer -= 1

        screen.blit(end_text, end_text_box)
        pygame.draw.rect(screen, floor_color, (0, world_h - floor, world_w, floor))

        pygame.display.update()
        clock.tick(FPS)    